import unittest
from docopt import docopt
from pyraider import cli

doc = cli.__doc__

class TestCLI(unittest.TestCase):
    def test_go(self):
        args = docopt(doc,["go"])
        self.assertEqual(args["go"], True)
    
    def test_validate(self):
        args = docopt(doc,["validate"])
        self.assertEqual(args["validate"], True)
        
    def test_fix(self):
        args = docopt(doc,["fix"])
        self.assertEqual(args["fix"], True)
    
    def test_autofix(self):
        args = docopt(doc,["autofix"])
        self.assertEqual(args["autofix"], True)
    

if __name__ == '__main__':
    unittest.main()


from beautifultable import BeautifulTable
import colored
from colored import stylize
import csv
import json
from json2html import *
import os
import pickle
from pkg_resources import parse_version
import pkg_resources
import subprocess
import sys
import ssl
import time

lib_path = os.path.abspath(os.path.join('..'))
sys.path.append(lib_path)
try:
    from urllib2 import Request, urlopen
except ImportError:
    from urllib.request import Request, urlopen, urlretrieve


def download_progress(count, block_size, total_size):
    global start_time
    if count == 0:
        start_time = time.time()
        return
    duration = time.time() - start_time
    progress_size = int(count * block_size)
    speed = int(progress_size / (1024 * duration))
    percent = int(count * block_size * 100 / total_size)
    sys.stdout.write("\r...%d%%, %d MB, %d KB/s, %d seconds passed" %
                    (percent, progress_size / (1024 * 1024), speed, duration))
    sys.stdout.flush()

def export_to_json(data_dict, export_file_path):
    """
        Export vulnerable data into a JSON file
    """
    if len(data_dict.get('pyraider'))>0:
        result_path = ''
        if export_file_path == '.':
            result_path = 'result.json'
            print(stylize('result.json has been exported in the current directory', colored.fg("green")))
        elif export_file_path:
            result_path = export_file_path
            filename = os.path.basename(result_path)
            print(stylize('{0} has been exported in {1} directory'.format(filename, result_path), colored.fg("green")))
        else:
            result_path = 'result.json'
            print(stylize('result.json has been exported in the current directory', colored.fg("green")))
        
        with open(result_path, 'w') as fp:
            json.dump(data_dict, fp, indent=4)

def export_to_html(data_dict, export_file_path):
    """
        Export vulnerable data into a JSON file
    """
    if len(data_dict.get('pyraider'))>0:
        result_path = ''
        if export_file_path == '.':
            result_path = 'result.html'
            print(stylize('result.html has been exported in the current directory', colored.fg("green")))
        elif export_file_path:
            result_path = export_file_path
            filename = os.path.basename(result_path)
            print(stylize('{0} has been exported in {1} directory'.format(filename, result_path), colored.fg("green")))
        else:
            result_path = 'result.html'
            print(stylize('result.html has been exported in the current directory', colored.fg("green")))
        data = json2html.convert(json = data_dict, table_attributes="id=\"info-table\" class=\"table table-striped table-bordered table-hover\"")
        header = """<html>
    <head>
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css" integrity="sha384-ggOyR0iXCbMQv3Xipma34MD+dH/1fQ784/j6cY/iJTQUOhcWr7x9JvoRxT2MZw1T" crossorigin="anonymous">
    </head>
    <div class="jumbotron jumbotron-fluid">
    <div class="container">
        <h1 class="display-4">PyRaider Result</h1>    
    </div>
    </div>
    <hr>
    <br>
    </html>"""
        with open(result_path, 'w') as fp:
            fp.write(header) 
            fp.write(data) 


def export_to_csv(data_dict, export_file_path):
    """
        Export vulnerable data into a CSV file
    """
    if len(data_dict.get('pyraider'))>0:
        result_path = ''
        if export_file_path == '.':
            result_path = 'result.csv'
            print(stylize('result.csv has been exported in the current directory', colored.fg("green")))
        elif export_file_path:
            result_path = export_file_path
            filename = os.path.basename(result_path)
            print(stylize('{0} has been exported in {1} directory'.format(filename, result_path), colored.fg("green")))
        else:
            result_path = 'result.csv'
            print(stylize('result.csv has been exported in the current directory', colored.fg("green")))
        with open(result_path, 'w') as f:
            writer = csv.DictWriter(f,
                                    fieldnames=['Package', 'Current Version', 'Description', 'Severity', 'CWE', 'CVE',
                                                'Update Version'])
            writer.writeheader()
            for k,v in data_dict.items():
                for data in v:
                    for k, v in data.items():
                        writer.writerow(
                            {"Package": k, "Current Version": v.get('current_version'), "Description": v.get('decription'),
                            'Severity': v.get('severity'), 'CWE': v.get('cwe'),
                            'CVE': v.get('cve'), 'Update Version': v.get('update_to')})


def show_vulnerablities(data_dict):
    """
        Render Vulnerable data into a terminal table
    """   
    for k, v in data_dict.items():
        parent_table = BeautifulTable()
        parent_table.rows.append(['Package', k])
        if v.get('severity') == 'HIGH':
            parent_table.rows.append(
                ["Severity", stylize(v.get('severity'), colored.fg("red"))])
        elif v.get('severity') == 'MEDIUM':
            parent_table.rows.append(["Severity", stylize(
                v.get('severity'), colored.fg("yellow"))])
        elif v.get('severity') == 'LOW':
            parent_table.rows.append(
                ["Severity", stylize(v.get('severity'), colored.fg("blue"))])
        else:
            parent_table.rows.append(
                ["Severity", stylize(v.get('severity'), colored.fg("blue"))])
        parent_table.rows.append(['CWE', v.get('cwe')])
        parent_table.rows.append(['CVE', v.get('cve')])        
        if v.get('current_version') < v.get('update_to'):
            parent_table.rows.append(['Current version', stylize(
                v.get('current_version'), colored.fg("red"))])
        else:
            parent_table.rows.append(['Current version', stylize(
                v.get('current_version'), colored.fg("green"))])
        if v.get('current_version') == v.get('update_to'):
            parent_table.rows.append(['Update To', stylize(
                'Package is up to date', colored.fg("green"))])
        else:
            parent_table.rows.append(['Update To', stylize(
                v.get('update_to'), colored.fg("green"))])
        parent_table.rows.append(['Description', v.get('description')])
        if v.get('dependency', None):
            parent_table.rows.append(['Dependency Of', v.get('dependency')])
        parent_table.rows.append(['Resolve', "pip install {0}=={1}".format(k,v.get('update_to'))])
        parent_table.rows.append(['More Info', "https://nvd.nist.gov/vuln/detail/{0}".format(v.get('cve'))])
        print('\n')
        print(parent_table)


def show_secure_packages(data_dict):
    """
        Render Vulnerable data into a terminal table
    """
    for secure in data_dict:
        for k, v in secure.items():
            parent_table = BeautifulTable()
            parent_table.rows.append(['Package', k])
            parent_table.rows.append(['Current version', stylize(
                v.get('current_version'), colored.fg("green"))])
            parent_table.rows.append(['Status', stylize(
                'No known security vulnerabilities found', colored.fg("green"))])
            print('\n')
            print(parent_table)


def render_package_update_report(data_dict):
    """
        Render package and latest version
    """
    print("\n")
    for k, v in data_dict.items():
        parent_table = BeautifulTable()
        parent_table.rows.append(['Package', k])
        if v.get('current_version') !=None:
            if v.get('current_version') < v.get('update_to'):
                parent_table.rows.append(['Current version', stylize(
                    v.get('current_version'), colored.fg("red"))])
            else:
                parent_table.rows.append(['Current version', stylize(
                    v.get('current_version'), colored.fg("green"))])
            if v.get('current_version') == v.get('update_to'):
                parent_table.rows.append(['Update To', stylize(
                    'Package is up to date', colored.fg("green"))])
            else:
                parent_table.rows.append(['Update To', stylize(
                    v.get('update_to'), colored.fg("green"))])
        else:
             parent_table.rows.append(['Latest Version', stylize(
                    v.get('update_to'), colored.fg("green"))])

        print('\n')
        print(parent_table)


def get_info_from_pypi(packages):
    """
        Get latest package version
    """
    ssl._create_default_https_context = ssl._create_unverified_context
    url = 'https://pypi.python.org/pypi/{0}/json'.format(packages)
    headers = {'Accept': 'application/json'}
    req = Request(url=url, headers=headers)
    resp = urlopen(req)
    if resp.code == 200:
        info = resp.read()
        decode_data = info.decode()
        info_data = json.loads(decode_data)
        latest_version = sorted(info_data["releases"], key=parse_version)
        return latest_version[-1]
    else:
        return 'Unexpected error'


def validate_version(packages, current_version):
    """
        Create a dict with current, and update version data
    """
    data_dict = {}
    validated_data = get_info_from_pypi(packages)
    data_dict[packages] = {}
    if current_version != None:
        data_dict[packages]['current_version'] = current_version
    data_dict[packages]['update_to'] = validated_data
    return data_dict


def scan_vulnerabilities():
    """
        Read from database
    """
    this_dir, _ = os.path.split(__file__)
    data_path = os.path.join(this_dir, 'resource.json')
    if os.path.exists(data_path):
        f = open(data_path)
        data = json.load(f)
        return data
    else:
        print(stylize('Downloading resources to scan the packages, It may take some time to download  .....', colored.fg("green")))
        ssl._create_default_https_context = ssl._create_unverified_context
        url = 'https://pyraider-source-data.s3-us-west-2.amazonaws.com/resource.pickle'
        try:
            urlretrieve(url, data_path, download_progress)
        except Exception as e:
            print(stylize('There is some error. You need to enable `https://pyraider-source-data.s3-us-west-2.amazonaws.com/` URL to download database',
                          colored.fg("red")))
        data = pickle.load(open(data_path, 'rb'))
        print(stylize('\nResource has been successfully downloaded', colored.fg("green")))
        return data

def scan_light_vulnerabilities():
    """
        Read from database
    """
    this_dir, _ = os.path.split(__file__)
    data_path = os.path.join(this_dir, 'resource_light.json')
    if os.path.exists(data_path):
        f = open(data_path)
        data = json.load(f)
        return data
    else:
        print(stylize('Downloading resources to scan the packages, It may take some time to download  .....', colored.fg("green")))
        ssl._create_default_https_context = ssl._create_unverified_context
        url = 'https://pyraider-source-data.s3-us-west-2.amazonaws.com/resource_light.json'
        try:
            urlretrieve(url, data_path, download_progress)
        except Exception as e:
            print(stylize('There is some error. You need to enable `https://pyraider-source-data.s3-us-west-2.amazonaws.com/` URL to download database',
                          colored.fg("red")))
        f = open(data_path)
        data = json.load(f)
        print(stylize('\nResource has been successfully downloaded', colored.fg("green")))
        return data


def check_latestdb():
    """
        check and download the latest database
    """
    this_dir, _ = os.path.split(__file__)
    data_path = os.path.join(this_dir, 'resource_light.json')
    if os.path.exists(data_path):
        os.remove(data_path)
    print(stylize('Downloading resources to scan the packages, It may take some time to download  .....', colored.fg("green")))
    ssl._create_default_https_context = ssl._create_unverified_context
    url = 'https://pyraider-source-data.s3-us-west-2.amazonaws.com/resource_light.json'
    try:
        urlretrieve(url, data_path, download_progress)
    except Exception as e:
        print(stylize('There is an error. You need to enable `https://pyraider-source-data.s3-us-west-2.amazonaws.com/` URL to download database',
                        colored.fg("red")))
    if os.path.exists(data_path):
        print(stylize('Resource database successfully downloaded and its last updated on March 2022', colored.fg("green")))


def show_dependencies(vul_pkg_name):
    pkg = pkg_resources.working_set.by_key[vul_pkg_name]
    data_dict = []
    if len(pkg.requires()) > 0:
        for vuls in pkg.requires():
            data = {}
            random_package_name = str(vuls)        
            package_name = pkg_resources.Requirement.parse(random_package_name).key
            package_version = pkg_resources.get_distribution(package_name).version
            name = f"{package_name}:{package_version}"
            data[name] = vul_pkg_name
            data_dict.append(data)
    return data_dict


def scan_severity(data, req_name, req_version, sev):
    """
        Scan High vulnerable library
    """
    data_dict = {}
    for k, v in data.items():
        if k.lower() == req_name:
            validated_version = get_info_from_pypi(k.lower())
            for vuls in v.get('info'):
                if vuls.get('sev') == sev:                    
                    if vuls.get('version'):
                        if req_version <= vuls.get('version') or vuls.get('version') <= req_version:
                            data_dict[k] = {}
                            data_dict[k]['current_version'] = req_version
                            data_dict[k]['update_to'] = validated_version
                            data_dict[k]['cwe'] = vuls.get('cwe')
                            data_dict[k]['cve'] = vuls.get('cve')
                            data_dict[k]['severity'] = vuls.get('sev')
                            if vuls.get('description'):
                                data_dict[k]['description'] = vuls.get('description')
    return data_dict


def scan_vulnerable_data(data, req_name, req_version,sev):
    """
        Scan vulnerable library
    """
    if sev:
        data_dict = scan_severity(data, req_name, req_version, sev)
        return data_dict
    else:
        data_dict = {}
        for k, v in data.items():
            if k.lower() == req_name:
                validated_version = get_info_from_pypi(k.lower())
                for vuls in v.get('info'):
                    if vuls.get('version'):
                        if req_version <= vuls.get('version') or vuls.get('version') <= req_version:
                            data_dict[k] = {}
                            data_dict[k]['current_version'] = req_version
                            data_dict[k]['update_to'] = validated_version
                            data_dict[k]['cwe'] = vuls.get('cwe')
                            data_dict[k]['cve'] = vuls.get('cve')
                            data_dict[k]['severity'] = vuls.get('sev')
                            if vuls.get('description'):
                                data_dict[k]['description'] = vuls.get('description')
        return data_dict


def query_yes_no(question, default="yes"):
    """
        Question prompt tag
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: {0}".format(default))
    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            if valid[choice]:
                print("")
            else:
                print("")
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")

                             
def check_installation(question, default="yes"):
    """
        Question prompt tag
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: {0}".format(default))
    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            if valid[choice]:
                print("")
            else:
                print("")
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")
    return valid[default]


def fix(data_dict):
    """
        Update latest version one by one
    """   
    pip_question = "Do you want to use pip to install packages?"
    pipenv_question = "Do you want to use pipenv to install packages?"
    conda_question = "Do you want to use conda to install packages?"
    check_is_pip = check_installation(pip_question)    
    is_install = False
    is_pip = False
    is_pipenv = False
    is_conda = False
    if check_is_pip == True:
        is_pip = True
        is_install = True
    elif is_pip==False:
        check_is_pipenv = check_installation(pipenv_question)
        if check_is_pipenv:
            is_pipenv = True
            is_install = True
    elif is_pip==False and is_pipenv==False:
        check_is_conda = check_installation(conda_question)
        if check_is_conda:
            is_conda = True
            is_install = True
    if is_install:
        for data in data_dict:
            for k, v in data.items():
                if v.get('current_version') < v.get('update_to'):
                    question = "Do you want to update {0} pacakge from {1} to {2} version?".format(k,v.get('current_version'), v.get('update_to'))
                    answers = query_yes_no(question)
                    if answers == True:
                        if is_pip:
                            installing = subprocess.call(
                                ['pip', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0}=={1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                        elif is_pipenv:
                            installing = subprocess.call(
                                ['pipenv', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0}=={1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                            print(
                                stylize("Pipfile has been updated successfully!!!"), colored.fg("green"))                    
                        elif is_conda:
                            installing = subprocess.call(
                                ['pip', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0}=={1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                        else:
                            installing = subprocess.call(
                                ['pip', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0}=={1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                else:
                    print(stylize("{0} is already up-to date to {1} version".format(k,
                                                                            v.get('update_to')), colored.fg("green")))
    else:
        print(stylize('You havent selected any of the option', colored.fg("green")))


def auto_fix_all(data_dict):
    """
        Update all packages 
    """
    ans = 'Are you sure want to update all the packages, It might affect other packages?'
    answers = query_yes_no(ans)
    if answers == True:
        pip_question = "Do you want to use pip to install packages?"
        pipenv_question = "Do you want to use pipenv to install packages?"
        conda_question = "Do you want to use conda to install packages?"
        check_is_pip = check_installation(pip_question)    
        is_install = False
        is_pip = False
        is_pipenv = False
        is_conda = False
        if check_is_pip == True:
            is_pip = True
            is_install = True
        elif is_pip==False:
            check_is_pipenv = check_installation(pipenv_question)
            if check_is_pipenv:
                is_pipenv = True
                is_install = True
        elif is_pip==False and is_pipenv==False:
            check_is_conda = check_installation(conda_question)
            if check_is_conda:
                is_conda = True
                is_install = True
        if is_install:
            for vul in data_dict:
                for k, v in vul.items():
                    if v.get('current_version') < v.get('update_to'):
                        if is_pip:
                            installing = subprocess.call(
                                ['pip', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0}=={1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                        elif is_pipenv:
                            installing = subprocess.call(
                                ['pipenv', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0}=={1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                            print(
                                stylize("Pipfile has been updated successfully!!!"), colored.fg("green"))                    
                        elif is_conda:
                            installing = subprocess.call(
                                ['pip', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0} == {1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                        else:
                            installing = subprocess.call(
                                ['pip', 'install', "{0}=={1}".format(k, v.get('update_to'))])
                            print(installing)
                            print(stylize("{0}=={1} version has been installed successfully!!!".format(
                                k, v.get('update_to')), colored.fg("green")))
                    else:
                        print(stylize("{0} is already up to date to {1} version".format(
                            k, v.get('update_to')), colored.fg("green")))
        else:
            print(stylize('You havent selected any of the option', colored.fg("green")))

# End-Of-File



"""
Usage:
  pyraider go [-d | -s [high|medium|low] | -e <format> | -e <format>  <exportFileName> | -e <format> -s [high|medium|low] | -e <format>  <exportFileName> -s [high|medium|low] | -d -s [high|medium|low] | -d -e <format> | -d -e <format>  <exportFileName> | -d -e <format> -s [high|medium|low] | -d -e <format> <exportFileName> -s [high|medium|low]]
  pyraider check -f <filename> [-d | -e <format> | <exportFileName> | -e <format>  <exportFileName> | -s [high|medium|low] | -e <format> -s [high|medium|low] |  -e <format>  <exportFileName> -s [high|medium|low]]
  pyraider validate [-p <package> | -f <filename>]
  pyraider fix [-d | -s [high|medium|low] | -d -s [high|medium|low]]
  pyraider autofix [-d | -s [high|medium|low] | -d -s [high|medium|low]]
  pyraider updatedb [-d]

Examples:
  pyraider go
  pyraider go -s high
  pyraider go -e json
  pyraider go -e json -s high
  pyraider check -f requirements.txt
  pyraider check -f requirements.txt -e json result.json
  pyraider check -f requirements.txt -e json result.json -s high
  pyraider validate
  pyraider validate -p django==1.11.13
  pyraider validate -f requirements.txt
  pyraider fix
  pyraider fix -s high
  pyraider autofix
  pyraider autofix -s high
  pyraider updatedb

Options:
  -h, --help
  -v, --version
"""

from docopt import docopt
import os
from main_pyraider import read_from_file, read_from_env, check_new_version, \
    fix_packages, auto_fix_all_packages, update_db

logo = """
  _____       _____       _     _
 |  __ \     |  __ \     (_)   | |
 | |__) |   _| |__) |__ _ _  __| | ___ _ __
 |  ___/ | | |  _  // _` | |/ _` |/ _ \ '__|
 | |   | |_| | | \ \ (_| | | (_| |  __/ |
 |_|    \__, |_|  \_\__,_|_|\__,_|\___|_|
         __/ |
        |___/

by RaiderSource version 1.0.15
"""


def find_file(name, path):
    """
        Find requirements.txt file
    """
    for root, dirs, files in os.walk(path):
        if name in files:
            return os.path.join(root, name)


def main():
    print(logo)
    arguments = docopt(__doc__, version='1.0.15')
    if arguments.get('check'):
        if arguments.get('high'):
            if arguments.get('<filename>') and not arguments.get('<exportFileName>') and not arguments.get('<format>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'),sev='HIGH')
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('-d') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
                try:
                    read_from_file(arguments.get('<filename>'), deep_scan=True, sev='HIGH')
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('<format>') and not arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), ".", sev='HIGH')
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('<format>') and arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), arguments.get('<exportFileName>'),sev='HIGH')
                except Exception as e:
                    exit(1)
        elif arguments.get('medium'):
            if arguments.get('<filename>') and not arguments.get('<exportFileName>') and not arguments.get('<format>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'),sev='MEDIUM')
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('-d') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
                try:
                    read_from_file(arguments.get('<filename>'), deep_scan=True, sev='MEDIUM')
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('<format>') and not arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), ".", sev='MEDIUM')
                except Exception as e:
                    exit(1)
            elif arguments.get('<format>') and arguments.get('<filename>') and arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), arguments.get('<exportFileName>'),sev='MEDIUM')
                except Exception as e:
                    exit(1)
        elif arguments.get('low'):
            if arguments.get('<filename>') and not arguments.get('<exportFileName>') and not arguments.get('<format>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'),sev='LOW')
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('-d') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
                try:
                    read_from_file(arguments.get('<filename>'), deep_scan=True, sev='LOW')
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('<format>') and not arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), ".", sev='LOW')
                except Exception as e:
                    exit(1)
            elif arguments.get('<format>') and arguments.get('<filename>') and arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), arguments.get('<exportFileName>'),sev='LOW')
                except Exception as e:
                    exit(1)
        else:
            if arguments.get('<filename>') and not arguments.get('<exportFileName>') and not arguments.get('<format>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'))
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('-d') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
                try:
                    read_from_file(arguments.get('<filename>'), deep_scan=True)
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>') and arguments.get('<format>') and not arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), ".")
                except Exception as e:
                    exit(1)
            elif arguments.get('<filename>')  and arguments.get('<format>') and arguments.get('<filename>') and arguments.get('<exportFileName>') and not arguments.get('-d'):
                try:
                    read_from_file(arguments.get('<filename>'), arguments.get('<format>'), arguments.get('<exportFileName>'))
                except Exception as e:
                    exit(1)
    if arguments.get('go'):  
        if arguments.get('-d') and arguments.get('<format>') and arguments.get('<exportFileName>') and arguments.get('high'):
            try:
                read_from_env(arguments.get('<format>'),arguments.get('<exportFileName>'), deep_scan=True, sev='HIGH')
            except Exception as e:
                exit(1)
        if arguments.get('-d') and arguments.get('<format>') and not arguments.get('<exportFileName>') and arguments.get('high'):
            try:
                read_from_env(arguments.get('<format>'), deep_scan=True, sev='HIGH')
            except Exception as e:
                exit(1)
        elif arguments.get('-d') and arguments.get('<format>') and arguments.get('<exportFileName>') and arguments.get('medium'):
            try:
                read_from_env(arguments.get('<format>'),arguments.get('<exportFileName>'), deep_scan=True, sev='MEDIUM')
            except Exception as e:
                exit(1)
        elif arguments.get('-d') and arguments.get('<format>') and not arguments.get('<exportFileName>') and arguments.get('medium'):
            try:
                read_from_env(arguments.get('<format>'), deep_scan=True, sev='MEDIUM')
            except Exception as e:
                exit(1)
        elif arguments.get('-d') and arguments.get('<format>') and arguments.get('<exportFileName>') and arguments.get('low'):
            try:
                read_from_env(arguments.get('<format>'),arguments.get('<exportFileName>'), deep_scan=True, sev='LOW')
            except Exception as e:
                exit(1)  
        elif arguments.get('-d') and arguments.get('<format>') and not arguments.get('<exportFileName>') and arguments.get('low'):
            try:
                read_from_env(arguments.get('<format>'), deep_scan=True, sev='LOW')
            except Exception as e:
                exit(1)    
        elif arguments.get('-d') and arguments.get('<format>') and arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'),arguments.get('<exportFileName>'), deep_scan=True)
            except Exception as e:
                exit(1)    
        elif arguments.get('-d') and arguments.get('<format>') and not arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'), deep_scan=True)
            except Exception as e:
                exit(1)
        elif arguments.get('-d') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
            try:
                read_from_env(deep_scan=True)
            except Exception as e:
                exit(1)                      
        elif arguments.get('high') and arguments.get('<format>') and arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'),arguments.get('<exportFileName>'), sev='HIGH')
            except Exception as e:
                exit(1)
        elif arguments.get('high') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
            try:
                read_from_env(sev='HIGH')
            except Exception as e:
                exit(1)
        elif arguments.get('high') and arguments.get('<format>') and not arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'), sev='HIGH')
            except Exception as e:
                exit(1)    
        elif arguments.get('medium') and arguments.get('<format>') and arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'), arguments.get('<exportFileName>'), sev='MEDIUM')
            except Exception as e:
                exit(1)    
        elif arguments.get('medium') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
            try:
                read_from_env(sev='MEDIUM')
            except Exception as e:
                exit(1)
        elif arguments.get('medium') and arguments.get('<format>') and not arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'), sev='MEDIUM')
            except Exception as e:
                exit(1)  
        elif arguments.get('low') and arguments.get('<format>') and arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'), arguments.get('<exportFileName>'), sev='LOW')
            except Exception as e:
                exit(1)              
        elif arguments.get('low') and not arguments.get('<exportFileName>') and not arguments.get('<format>'):
            try:
                read_from_env(sev='LOW')
            except Exception as e:
                exit(1)
        elif arguments.get('low') and arguments.get('<format>') and not arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'), sev='LOW')
            except Exception as e:
                exit(1)          
        elif arguments.get('<format>') and not arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'))
            except Exception as e:
                exit(1)  
        elif arguments.get('<format>') and arguments.get('<exportFileName>'):
            try:
                read_from_env(arguments.get('<format>'), arguments.get('<exportFileName>'))
            except Exception as e:
                exit(1)        
        else:
            try:
                read_from_env()
            except Exception as e:
                exit(1)
    if arguments.get('validate'):
        try:
            if arguments.get('<package>'):
                check_new_version(arguments.get('<package>'), vpackage=True)
            elif arguments.get('<filename>'):
                check_new_version(arguments.get('<filename>'))
            else:
                check_new_version()
        except Exception as e:
            exit(1)
    if arguments.get('fix'):
        if arguments.get('-d') and arguments.get('high'):
            fix_packages(sev='HIGH', deep_scan=True)
        elif arguments.get('-d') and arguments.get('medium'):
            fix_packages(sev='MEDIUM', deep_scan=True)
        elif arguments.get('-d') and arguments.get('low'):
            fix_packages(sev='LOW', deep_scan=True)
        elif arguments.get('high'):
            fix_packages(sev='HIGH')
        elif arguments.get('medium'):
            fix_packages(sev='MEDIUM')
        elif arguments.get('low'):
            fix_packages(sev='LOW')
        elif arguments.get('-d'):
            fix_packages(deep_scan=True)
        else:
            try:
                fix_packages()
            except Exception as e:
                exit(1)
    if arguments.get('autofix'):
        if arguments.get('-d') and arguments.get('high'):
            auto_fix_all_packages(sev='HIGH', deep_scan=True)
        elif arguments.get('-d') and arguments.get('medium'):
            auto_fix_all_packages(sev='MEDIUM', deep_scan=True)
        elif arguments.get('-d') and arguments.get('low'):
            auto_fix_all_packages(sev='LOW', deep_scan=True)
        elif arguments.get('high'):
            auto_fix_all_packages(sev='HIGH')
        elif arguments.get('medium'):
            auto_fix_all_packages(sev='MEDIUM')
        elif arguments.get('low'):
            auto_fix_all_packages(sev='LOW')
        elif arguments.get('-d'):
            auto_fix_all_packages(deep_scan=True)
        else:
            try:
                auto_fix_all_packages()
            except Exception as e:
                exit(1)
    if arguments.get('updatedb'):
        try:
            update_db()
        except Exception as e:
            exit(1)


if __name__ == "__main__":
    main()

# End-Of-File


import colored
from colored import stylize
import json
import os
import pkg_resources
from utils import export_to_csv, export_to_json, show_vulnerablities, \
    render_package_update_report, scan_vulnerabilities, scan_vulnerable_data, \
    validate_version, fix, auto_fix_all, show_secure_packages, check_latestdb, \
    scan_light_vulnerabilities, export_to_html, show_dependencies


def read_from_env(export_format=None, export_file_path=None, deep_scan=False, sev=None):
    """
        Collect requirments from env and scan and show reports
    """
    print(stylize('Started Scanning .....', colored.fg("green")))
    print('\n')
    data_dict = {}
    secure_data_dict = []
    data_dict['pyraider'] = []
    data_dict['version'] = '1.0.15'
    vul_package_count = 0
    if deep_scan:
        data = scan_vulnerabilities()
    else:
        data = scan_light_vulnerabilities()
    dependency_list = []
    for package in pkg_resources.working_set:
        req_name = package.key.lower()
        req_version = package.version
        dependent = None
        dependency = show_dependencies(req_name)
        if len(dependency) > 0:
            dependency_list.append(dependency)
        scanned_data = scan_vulnerable_data(data, req_name, req_version, sev)
        flat_list = [item for sublist in dependency_list for item in sublist]
        if bool(scanned_data):
            vul_package_count += 1
            if len(flat_list) > 0:
                name = f"{req_name}:{req_version}"
                try:
                    dependent = next(d.get(name, None)
                                     for i, d in enumerate(flat_list) if name in d)
                    scanned_data[req_name]['dependency'] = dependent
                except Exception as e:
                    scanned_data[req_name]['dependency'] = req_name
            if not export_format:
                show_vulnerablities(scanned_data)
            if export_format == 'json':
                data_dict['pyraider'].append(scanned_data)
            elif export_format == 'csv':
                data_dict['pyraider'].append(scanned_data)
            elif export_format == 'html':
                data_dict['pyraider'].append(scanned_data)
            if not export_format:
                show_secure_packages(secure_data_dict)
        if not export_format:
            show_secure_packages(secure_data_dict)
    if export_format == 'json':
        export_to_json(data_dict, export_file_path)
    elif export_format == 'csv':
        export_to_csv(data_dict, export_file_path)
    elif export_format == 'html':
        export_to_html(data_dict, export_file_path)
    if vul_package_count == 0:
        print(stylize('No known vulnerabilities found', colored.fg("green")))


def check_new_version(to_scan_file=None, vpackage=False):
    """
        Check latest version from requirements.txt file
    """
    if to_scan_file:
        _, file_extension = os.path.splitext(to_scan_file)
        if file_extension == '.lock':
            with open(to_scan_file) as fp:
                line = json.loads(fp.read())
                for k, v in line['default'].items():
                    validated_data = validate_version(
                        k.lower(), v['version'].split("==")[1])
                    render_package_update_report(validated_data)
        if file_extension == '.txt':
            with open(to_scan_file) as fp:
                line = fp.readline()
                cnt = 1
                while line:
                    req = line.strip().split('==')
                    if len(req) == 2:
                        req_name = req[0].lower()
                        req_version = req[1]
                        validated_data = validate_version(
                            req_name, req_version)
                        render_package_update_report(validated_data)
                    line = fp.readline()
                    cnt += 1
    if to_scan_file and vpackage:
        req = to_scan_file.strip().split('==')
        if len(req) == 2:
            req_name = req[0].lower()
            req_version = req[1]
            validated_data = validate_version(
                req_name, req_version)
            render_package_update_report(validated_data)
        else:
            validated_data = validate_version(req[0].lower(), None)
            render_package_update_report(validated_data)

    if to_scan_file == None and vpackage == False:
        dists = [d for d in pkg_resources.working_set]
        for pkg in dists:
            convert_str = str(pkg)
            package = convert_str.split()
            req_name = package[0].lower()
            req_version = package[1]
            validated_data = validate_version(req_name, req_version)
            render_package_update_report(validated_data)


def read_from_file(to_scan_file, export_format=None, export_file_path=None, deep_scan=False, sev=None):
    """
        Read requirents from requirements.txt file and also we can generate a JSON and CSV report.
    """
    print(stylize('Started Scanning .....', colored.fg("green")))
    print('\n')
    data_dict = {}
    secure_data_dict = []
    data_dict['pyraider'] = []
    data_dict['version'] = '1.0.15'
    vul_package_count = 0
    _, file_extension = os.path.splitext(to_scan_file)
    if deep_scan:
        data = scan_vulnerabilities()
    else:
        data = scan_light_vulnerabilities()
    list_pkg = {}
    if file_extension == '.lock':
        with open(to_scan_file) as fp:
            line = json.loads(fp.read())
            for k, v in line['default'].items():
                req_name = k.lower()
                package_version = v['version'].split("==")
                req_version = package_version[1]
                list_pkg[req_name] = req_version
    if file_extension == '.txt':
        with open(to_scan_file) as fp:
            line = fp.readline()
            cnt = 1
            while line:
                package = line.strip()
                txt_req = package.split('==')
                if len(txt_req) == 2:
                    req_name = txt_req[0].lower()
                    req_version = txt_req[1]
                    list_pkg[req_name] = req_version
                line = fp.readline()
                cnt += 1
    for req_name, req_version in list_pkg.items():
        scanned_data = scan_vulnerable_data(data, req_name, req_version, sev)
        if bool(scanned_data):
            vul_package_count += 1
            if not export_format:
                show_vulnerablities(scanned_data)
            if export_format == 'json':
                data_dict['pyraider'].append(scanned_data)
            elif export_format == 'csv':
                data_dict['pyraider'].append(scanned_data)
            elif export_format == 'html':
                data_dict['pyraider'].append(scanned_data)

    if not export_format:
        show_secure_packages(secure_data_dict)
    if export_format == 'json':
        export_to_json(data_dict, export_file_path)
    elif export_format == 'csv':
        export_to_csv(data_dict, export_file_path)
    elif export_format == 'html':
        export_to_html(data_dict, export_file_path)
    if vul_package_count == 0:
        print(stylize('No known vulnerabilities found', colored.fg("green")))


def fix_packages(to_scan_file=None, is_pipenv=False, sev=None, deep_scan=False):
    """
        Update one by one packages
    """
    print(stylize('Started Scanning .....', colored.fg("green")))
    print('\n')
    if deep_scan:
        data = scan_vulnerabilities()
    else:
        data = scan_light_vulnerabilities()
    dists = [d for d in pkg_resources.working_set]
    data_list = []
    vul_package_count = 0
    for pkg in dists:
        convert_str = str(pkg)
        package = convert_str.split()
        req_name = package[0].lower()
        req_version = package[1]
        scanned_data = scan_vulnerable_data(data, req_name, req_version, sev)
        if bool(scanned_data):
            vul_package_count += 1
            data_list.append(scanned_data)
    if len(data_list) > 0:
        fix(data_list)
    if vul_package_count == 0:
        print(stylize('No known vulnerabilities found', colored.fg("green")))


def auto_fix_all_packages(to_scan_file=None, is_pipenv=False, sev=None, deep_scan=False):
    """
        Update all packages
    """
    print(stylize('Started Scanning .....', colored.fg("green")))
    print('\n')
    if deep_scan:
        data = scan_vulnerabilities()
    else:
        data = scan_light_vulnerabilities()
    all_packages = []
    vul_package_count = 0
    dists = [d for d in pkg_resources.working_set]
    for pkg in dists:
        convert_str = str(pkg)
        package = convert_str.split()
        req_name = package[0].lower()
        req_version = package[1]
        scanned_data = scan_vulnerable_data(data, req_name, req_version, sev)
        if bool(scanned_data):
            vul_package_count += 1
            all_packages.append(scanned_data)
    if len(all_packages) > 0:
        auto_fix_all(all_packages)
    if vul_package_count == 0:
        print(stylize('No known vulnerabilities found', colored.fg("green")))


def update_db():
    check_latestdb()
# End-of-file



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
    
setuptools.setup(
    name='pyraider',
    version='1.0.15',
    packages=setuptools.find_packages(),    
    author='Tilak Thimmappa',
    author_email='tilakthimmappa@gmail.com',
    description="Using PyRaider You can scan installed dependencies known security vulnerabilities. It uses publicly known exploits, vulnerabilities database.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points ={ 
        'console_scripts': [ 
            'pyraider = pyraider.cli:main'
        ] 
    }, 
    url="https://github.com/raidersource/pyraider",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3',
    keywords = ['SCA', 'pyraider', 'Source Composition Analysis', 'vulnerability scanner'],   # Keywords that define your package best
    install_requires=[
        'docopt',
        'beautifultable',
        'colored',
        'json2html'
    ],
)